@@ -1,8 +1,11 @@
+import sys
+
 from sympy.core import Symbol, Function, Float, Rational, Integer, I, Mul, Pow, Eq
-from sympy.functions import exp, factorial, sin
+from sympy.core.compatibility import PY3
+from sympy.functions import exp, factorial, factorial2, sin
 from sympy.logic import And
 from sympy.series import Limit
-from sympy.utilities.pytest import raises
+from sympy.utilities.pytest import raises, skip
 
 from sympy.parsing.sympy_parser import (
     parse_expr, standard_transformations, rationalize, TokenError,
@@ -19,7 +22,18 @@ def test_sympy_parser():
         '2+3j': 2 + 3*I,
         'exp(x)': exp(x),
         'x!': factorial(x),
+        'x!!': factorial2(x),
+        '(x + 1)! - 1': factorial(x + 1) - 1,
         '3.[3]': Rational(10, 3),
+        '.0[3]': Rational(1, 30),
+        '3.2[3]': Rational(97, 30),
+        '1.3[12]': Rational(433, 330),
+        '1 + 3.[3]': Rational(13, 3),
+        '1 + .0[3]': Rational(31, 30),
+        '1 + 3.2[3]': Rational(127, 30),
+        '.[0011]': Rational(1, 909),
+        '0.1[00102] + 1': Rational(366697, 333330),
+        '1.[0191]': Rational(10190, 9999),
         '10!': 3628800,
         '-(2)': -Integer(2),
         '[-1, -2, 3]': [Integer(-1), Integer(-2), Integer(3)],
@@ -56,6 +70,22 @@ def test_factorial_fail():
             assert True
 
 
+def test_repeated_fail():
+    inputs = ['1[1]', '.1e1[1]', '0x1[1]', '1.1j[1]', '1.1[1 + 1]',
+        '0.1[[1]]', '0x1.1[1]']
+
+    # All are valid Python, so only raise TypeError for invalid indexing
+    for text in inputs:
+        raises(TypeError, lambda: parse_expr(text))
+
+    inputs = ['0.1[', '0.1[1', '0.1[]']
+    for text in inputs:
+        raises((TokenError, SyntaxError), lambda: parse_expr(text))
+
+def test_repeated_dot_only():
+    assert parse_expr('.[1]') == Rational(1, 9)
+    assert parse_expr('1 + .[1]') == Rational(10, 9)
+
 def test_local_dict():
     local_dict = {
         'my_function': lambda x: x + 2
@@ -88,6 +118,22 @@ def test_issue_7663():
     e = '2*(x+1)'
     assert parse_expr(e, evaluate=0) == parse_expr(e, evaluate=False)
 
+def test_issue_10560():
+    inputs = {
+        '4*-3' : '(-3)*4',
+        '-4*3' : '(-4)*3',
+    }
+    for text, result in inputs.items():
+        assert parse_expr(text, evaluate=False) == parse_expr(result, evaluate=False)
+
+def test_issue_10773():
+    inputs = {
+    '-10/5': '(-10)/5',
+    '-10/-5' : '(-10)/(-5)',
+    }
+    for text, result in inputs.items():
+        assert parse_expr(text, evaluate=False) == parse_expr(result, evaluate=False)
+
 
 def test_split_symbols():
     transformations = standard_transformations + \
@@ -126,3 +172,21 @@ def test_convert_equals_signs():
     assert parse_expr("y = x", transformations=transformations) == Eq(y, x)
     assert parse_expr("(2*y = x) = False",
         transformations=transformations) == Eq(Eq(2*y, x), False)
+
+def test_unicode_names():
+    if not PY3:
+        skip("test_unicode_names can only pass in Python 3")
+
+    assert parse_expr(u'α') == Symbol(u'α')
+
+def test_python3_features():
+    # Make sure the tokenizer can handle Python 3-only features
+    if sys.version_info < (3, 6):
+        skip("test_python3_features requires Python 3.6 or newer")
+
+    assert parse_expr("123_456") == 123456
+    assert parse_expr("1.2[3_4]") == parse_expr("1.2[34]") == Rational(611, 495)
+    assert parse_expr("1.2[012_012]") == parse_expr("1.2[012012]") == Rational(400, 333)
+    assert parse_expr('.[3_4]') == parse_expr('.[34]') == Rational(34, 99)
+    assert parse_expr('.1[3_4]') == parse_expr('.1[34]') == Rational(133, 990)
+    assert parse_expr('123_123.123_123[3_4]') == parse_expr('123123.123123[34]') == Rational(12189189189211, 99000000)
