@@ -2,7 +2,8 @@ from sympy import (sin, cos, atan2, log, exp, gamma, conjugate, sqrt,
     factorial, Integral, Piecewise, Add, diff, symbols, S, Float, Dummy, Eq,
     Range, Catalan, EulerGamma, E, GoldenRatio, I, pi, Function, Rational, Integer, Lambda, sign)
 
-from sympy.codegen import For, Assignment
+from sympy.codegen import For, Assignment, aug_assign
+from sympy.codegen.ast import Declaration, Type, Variable, float32, float64, value_const, real, bool_, While
 from sympy.core.relational import Relational
 from sympy.logic.boolalg import And, Or, Not, Equivalent, Xor
 from sympy.printing.fcode import fcode, FCodePrinter
@@ -45,7 +46,7 @@ def test_fcode_Pow():
     assert fcode(x**0.5) == '      sqrt(x)'
     assert fcode(sqrt(x)) == '      sqrt(x)'
     assert fcode(sqrt(10)) == '      sqrt(10.0d0)'
-    assert fcode(x**-1.0) == '      1.0/x'
+    assert fcode(x**-1.0) == '      1d0/x'
     assert fcode(x**-2.0, 'y', source_format='free') == 'y = x**(-2.0d0)'  # 2823
     assert fcode(x**Rational(3, 7)) == '      x**(3.0d0/7.0d0)'
 
@@ -75,37 +76,70 @@ def test_fcode_functions():
     assert fcode(sin(x) ** cos(y)) == "      sin(x)**cos(y)"
 
 
+def test_case():
+    ob = FCodePrinter()
+    x,x_,x__,y,X,X_,Y = symbols('x,x_,x__,y,X,X_,Y')
+    assert fcode(exp(x_) + sin(x*y) + cos(X*Y)) == \
+                        '      exp(x_) + sin(x*y) + cos(X__*Y_)'
+    assert fcode(exp(x__) + 2*x*Y*X_**Rational(7, 2)) == \
+                        '      2*X_**(7.0d0/2.0d0)*Y*x + exp(x__)'
+    assert fcode(exp(x_) + sin(x*y) + cos(X*Y), name_mangling=False) == \
+                        '      exp(x_) + sin(x*y) + cos(X*Y)'
+    assert fcode(x - cos(X), name_mangling=False) == '      x - cos(X)'
+    assert ob.doprint(X*sin(x) + x_, assign_to='me') == '      me = X*sin(x_) + x__'
+    assert ob.doprint(X*sin(x), assign_to='mu') == '      mu = X*sin(x_)'
+    assert ob.doprint(x_, assign_to='ad') == '      ad = x__'
+    n, m = symbols('n,m', integer=True)
+    A = IndexedBase('A')
+    x = IndexedBase('x')
+    y = IndexedBase('y')
+    i = Idx('i', m)
+    I = Idx('I', n)
+    assert fcode(A[i, I]*x[I], assign_to=y[i], source_format='free') == (
+                                            "do i = 1, m\n"
+                                            "   y(i) = 0\n"
+                                            "end do\n"
+                                            "do i = 1, m\n"
+                                            "   do I_ = 1, n\n"
+                                            "      y(i) = A(i, I_)*x(I_) + y(i)\n"
+                                            "   end do\n"
+                                            "end do" )
+
+
 #issue 6814
 def test_fcode_functions_with_integers():
     x= symbols('x')
-    assert fcode(x * log(10)) == "      x*2.30258509299405d0"
-    assert fcode(x * log(10)) == "      x*2.30258509299405d0"
-    assert fcode(x * log(S(10))) == "      x*2.30258509299405d0"
-    assert fcode(log(S(10))) == "      2.30258509299405d0"
-    assert fcode(exp(10)) == "      22026.4657948067d0"
-    assert fcode(x * log(log(10))) == "      x*0.834032445247956d0"
-    assert fcode(x * log(log(S(10)))) == "      x*0.834032445247956d0"
+    log10_17 = log(10).evalf(17)
+    loglog10_17 = '0.8340324452479558d0'
+    assert fcode(x * log(10)) == "      x*%sd0" % log10_17
+    assert fcode(x * log(10)) == "      x*%sd0" % log10_17
+    assert fcode(x * log(S(10))) == "      x*%sd0" % log10_17
+    assert fcode(log(S(10))) == "      %sd0" % log10_17
+    assert fcode(exp(10)) == "      %sd0" % exp(10).evalf(17)
+    assert fcode(x * log(log(10))) == "      x*%s" % loglog10_17
+    assert fcode(x * log(log(S(10)))) == "      x*%s" % loglog10_17
 
 
 def test_fcode_NumberSymbol():
+    prec = 17
     p = FCodePrinter()
-    assert fcode(Catalan) == '      parameter (Catalan = 0.915965594177219d0)\n      Catalan'
-    assert fcode(EulerGamma) == '      parameter (EulerGamma = 0.577215664901533d0)\n      EulerGamma'
-    assert fcode(E) == '      parameter (E = 2.71828182845905d0)\n      E'
-    assert fcode(GoldenRatio) == '      parameter (GoldenRatio = 1.61803398874989d0)\n      GoldenRatio'
-    assert fcode(pi) == '      parameter (pi = 3.14159265358979d0)\n      pi'
+    assert fcode(Catalan) == '      parameter (Catalan = %sd0)\n      Catalan' % Catalan.evalf(prec)
+    assert fcode(EulerGamma) == '      parameter (EulerGamma = %sd0)\n      EulerGamma' % EulerGamma.evalf(prec)
+    assert fcode(E) == '      parameter (E = %sd0)\n      E' % E.evalf(prec)
+    assert fcode(GoldenRatio) == '      parameter (GoldenRatio = %sd0)\n      GoldenRatio' % GoldenRatio.evalf(prec)
+    assert fcode(pi) == '      parameter (pi = %sd0)\n      pi' % pi.evalf(prec)
     assert fcode(
-        pi, precision=5) == '      parameter (pi = 3.1416d0)\n      pi'
+        pi, precision=5) == '      parameter (pi = %sd0)\n      pi' % pi.evalf(5)
     assert fcode(Catalan, human=False) == (set(
-        [(Catalan, p._print(Catalan.evalf(15)))]), set([]), '      Catalan')
+        [(Catalan, p._print(Catalan.evalf(prec)))]), set([]), '      Catalan')
     assert fcode(EulerGamma, human=False) == (set([(EulerGamma, p._print(
-        EulerGamma.evalf(15)))]), set([]), '      EulerGamma')
+        EulerGamma.evalf(prec)))]), set([]), '      EulerGamma')
     assert fcode(E, human=False) == (
-        set([(E, p._print(E.evalf(15)))]), set([]), '      E')
+        set([(E, p._print(E.evalf(prec)))]), set([]), '      E')
     assert fcode(GoldenRatio, human=False) == (set([(GoldenRatio, p._print(
-        GoldenRatio.evalf(15)))]), set([]), '      GoldenRatio')
+        GoldenRatio.evalf(prec)))]), set([]), '      GoldenRatio')
     assert fcode(pi, human=False) == (
-        set([(pi, p._print(pi.evalf(15)))]), set([]), '      pi')
+        set([(pi, p._print(pi.evalf(prec)))]), set([]), '      pi')
     assert fcode(pi, precision=5, human=False) == (
         set([(pi, p._print(pi.evalf(5)))]), set([]), '      pi')
 
@@ -159,9 +193,9 @@ def test_inline_function():
     assert fcode(g(x)) == "      2*x"
     g = implemented_function('g', Lambda(x, 2*pi/x))
     assert fcode(g(x)) == (
-        "      parameter (pi = 3.14159265358979d0)\n"
+        "      parameter (pi = %sd0)\n"
         "      2*pi/x"
-    )
+    ) % pi.evalf(17)
     A = IndexedBase('A')
     i = Idx('i', symbols('n', integer=True))
     g = implemented_function('g', Lambda(x, x*(1 + x)*(2 + x)))
@@ -687,6 +721,31 @@ def test_fcode_For():
                    "      end do")
 
 
+def test_fcode_Declaration():
+    def check(expr, ref, **kwargs):
+        assert fcode(expr, standard=95, source_format='free', **kwargs) == ref
+
+    i = symbols('i', integer=True)
+    var1 = Variable.deduced(i)
+    dcl1 = Declaration(var1)
+    check(dcl1, "integer*4 :: i")
+
+
+    x, y = symbols('x y')
+    var2 = Variable(x, float32, value=42, attrs={value_const})
+    dcl2b = Declaration(var2)
+    check(dcl2b, 'real*4, parameter :: x = 42')
+
+    var3 = Variable(y, type=bool_)
+    dcl3 = Declaration(var3)
+    check(dcl3, 'logical :: y')
+
+    check(float32, "real*4")
+    check(float64, "real*8")
+    check(real, "real*4", type_aliases={real: float32})
+    check(real, "real*8", type_aliases={real: float64})
+
+
 def test_MatrixElement_printing():
     # test cases for issue #11821
     A = MatrixSymbol("A", 1, 3)
@@ -697,4 +756,18 @@ def test_MatrixElement_printing():
     assert(fcode(3 * A[0, 0]) == "      3*A(1, 1)")
 
     F = C[0, 0].subs(C, A - B)
-    assert(fcode(F) == "      ((-1)*B + A)(1, 1)")
+    assert(fcode(F) == "      (-B + A)(1, 1)")
+
+
+def test_aug_assign():
+    x = symbols('x')
+    assert fcode(aug_assign(x, '+', 1), source_format='free') == 'x = x + 1'
+
+
+def test_While():
+    x = symbols('x')
+    assert fcode(While(abs(x) > 1, [aug_assign(x, '-', 1)]), source_format='free') == (
+        'do while (abs(x) > 1)\n'
+        '   x = x - 1\n'
+        'end do'
+    )
