@@ -19,6 +19,7 @@ FisherZ
 Frechet
 Gamma
 GammaInverse
+Gumbel
 Gompertz
 Kumaraswamy
 Laplace
@@ -33,6 +34,7 @@ RaisedCosine
 Rayleigh
 ShiftedGompertz
 StudentT
+Trapezoidal
 Triangular
 Uniform
 UniformSum
@@ -45,12 +47,15 @@ from __future__ import print_function, division
 
 from sympy import (log, sqrt, pi, S, Dummy, Interval, sympify, gamma,
                    Piecewise, And, Eq, binomial, factorial, Sum, floor, Abs,
-                   Lambda, Basic)
+                   Lambda, Basic, lowergamma, erf, erfi, erfc, I, hyper, uppergamma,
+                   sinh, Ne)
+
 from sympy import beta as beta_fn
-from sympy import cos, exp, besseli
+from sympy import cos, sin, exp, besseli, besselj
 from sympy.stats.crv import (SingleContinuousPSpace, SingleContinuousDistribution,
         ContinuousDistributionHandmade)
 from sympy.stats.rv import _value_check
+from sympy.external import import_module
 import random
 
 oo = S.Infinity
@@ -73,6 +78,7 @@ __all__ = ['ContinuousRV',
 'Gamma',
 'GammaInverse',
 'Gompertz',
+'Gumbel',
 'Kumaraswamy',
 'Laplace',
 'Logistic',
@@ -86,6 +92,7 @@ __all__ = ['ContinuousRV',
 'Rayleigh',
 'StudentT',
 'ShiftedGompertz',
+'Trapezoidal',
 'Triangular',
 'Uniform',
 'UniformSum',
@@ -129,6 +136,7 @@ def ContinuousRV(symbol, density, set=Interval(-oo, oo)):
     dist = ContinuousDistributionHandmade(pdf, set)
     return SingleContinuousPSpace(symbol, dist).value
 
+
 def rv(symbol, cls, args):
     args = list(map(sympify, args))
     dist = cls(*args)
@@ -149,6 +157,15 @@ class ArcsinDistribution(SingleContinuousDistribution):
     def pdf(self, x):
         return 1/(pi*sqrt((x - self.a)*(self.b - x)))
 
+    def _cdf(self, x):
+        from sympy import asin
+        a, b = self.a, self.b
+        return Piecewise(
+            (S.Zero, x < a),
+            (2*asin(sqrt((x - a)/(b - a)))/pi, x <= b),
+            (S.One, True))
+
+
 def Arcsin(name, a=0, b=1):
     r"""
     Create a Continuous Random Variable with an arcsin distribution.
@@ -174,7 +191,7 @@ def Arcsin(name, a=0, b=1):
     Examples
     ========
 
-    >>> from sympy.stats import Arcsin, density
+    >>> from sympy.stats import Arcsin, density, cdf
     >>> from sympy import Symbol, simplify
 
     >>> a = Symbol("a", real=True)
@@ -186,6 +203,12 @@ def Arcsin(name, a=0, b=1):
     >>> density(X)(z)
     1/(pi*sqrt((-a + z)*(b - z)))
 
+    >>> cdf(X)(z)
+    Piecewise((0, a > z),
+            (2*asin(sqrt((-a + z)/(-a + b)))/pi, b >= z),
+            (1, True))
+
+
     References
     ==========
 
@@ -288,6 +311,9 @@ class BetaDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.betavariate(self.alpha, self.beta)
 
+    def _characteristic_function(self, t):
+        return hyper((self.alpha,), (self.alpha + self.beta,), I*t)
+
 
 def Beta(name, alpha, beta):
     r"""
@@ -328,7 +354,7 @@ def Beta(name, alpha, beta):
      alpha - 1         beta - 1
     z         *(-z + 1)
     ---------------------------
-         beta(alpha, beta)
+           B(alpha, beta)
 
     >>> expand_func(simplify(E(X, meijerg=True)))
     alpha/(alpha + beta)
@@ -398,7 +424,7 @@ def BetaPrime(name, alpha, beta):
      alpha - 1        -alpha - beta
     z         *(z + 1)
     -------------------------------
-           beta(alpha, beta)
+             B(alpha, beta)
 
     References
     ==========
@@ -477,6 +503,13 @@ class ChiDistribution(SingleContinuousDistribution):
     def pdf(self, x):
         return 2**(1 - self.k/2)*x**(self.k - 1)*exp(-x**2/2)/gamma(self.k/2)
 
+    def _characteristic_function(self, t):
+        k = self.k
+
+        part_1 = hyper((k/2,), (S(1)/2,), -t**2/2)
+        part_2 = I*t*sqrt(2)*gamma((k+1)/2)/gamma(k/2)
+        part_3 = hyper(((k+1)/2,), (S(3)/2,), -t**2/2)
+        return part_1 + part_2*part_3
 
 def Chi(name, k):
     r"""
@@ -596,6 +629,15 @@ class ChiSquaredDistribution(SingleContinuousDistribution):
         k = self.k
         return 1/(2**(k/2)*gamma(k/2))*x**(k/2 - 1)*exp(-x/2)
 
+    def _cdf(self, x):
+        k = self.k
+        return Piecewise(
+                (S.One/gamma(k/2)*lowergamma(k/2, x/2), x >= 0),
+                (0, True)
+        )
+
+    def _characteristic_function(self, t):
+        return (1 - 2*I*t)**(-self.k/2)
 
 def ChiSquared(name, k):
     r"""
@@ -623,7 +665,7 @@ def ChiSquared(name, k):
     ========
 
     >>> from sympy.stats import ChiSquared, density, E, variance
-    >>> from sympy import Symbol, simplify, combsimp, expand_func
+    >>> from sympy import Symbol, simplify, gammasimp, expand_func
 
     >>> k = Symbol("k", integer=True, positive=True)
     >>> z = Symbol("z")
@@ -633,7 +675,7 @@ def ChiSquared(name, k):
     >>> density(X)(z)
     2**(-k/2)*z**(k/2 - 1)*exp(-z/2)/gamma(k/2)
 
-    >>> combsimp(E(X))
+    >>> gammasimp(E(X))
     k
 
     >>> simplify(expand_func(variance(X)))
@@ -659,6 +701,11 @@ class DagumDistribution(SingleContinuousDistribution):
         p, a, b = self.p, self.a, self.b
         return a*p/x*((x/b)**(a*p)/(((x/b)**a + 1)**(p + 1)))
 
+    def _cdf(self, x):
+        p, a, b = self.p, self.a, self.b
+        return Piecewise(((S.One + (S(x)/b)**-a)**-p, x>=0),
+                    (S.Zero, True))
+
 
 def Dagum(name, p, a, b):
     r"""
@@ -687,7 +734,7 @@ def Dagum(name, p, a, b):
     Examples
     ========
 
-    >>> from sympy.stats import Dagum, density
+    >>> from sympy.stats import Dagum, density, cdf
     >>> from sympy import Symbol, simplify
 
     >>> p = Symbol("p", positive=True)
@@ -700,6 +747,10 @@ def Dagum(name, p, a, b):
     >>> density(X)(z)
     a*p*(z/b)**(a*p)*((z/b)**a + 1)**(-p - 1)/z
 
+    >>> cdf(X)(z)
+    Piecewise(((1 + (z/b)**(-a))**(-p), z >= 0), (0, True))
+
+
     References
     ==========
 
@@ -711,6 +762,7 @@ def Dagum(name, p, a, b):
 #-------------------------------------------------------------------------------
 # Erlang distribution ----------------------------------------------------------
 
+
 def Erlang(name, k, l):
     r"""
     Create a continuous random variable with an Erlang distribution.
@@ -750,16 +802,16 @@ def Erlang(name, k, l):
      k  k - 1  -l*z
     l *z     *e
     ---------------
-        gamma(k)
+        Gamma(k)
 
     >>> C = cdf(X, meijerg=True)(z)
     >>> pprint(C, use_unicode=False)
-    /     -2*I*pi*k                       -2*I*pi*k
-    |  k*e         *lowergamma(k, 0)   k*e         *lowergamma(k, l*z)
-    |- ----------------------------- + -------------------------------  for z >= 0
-    <           gamma(k + 1)                     gamma(k + 1)
+    /   -2*I*pi*k
+    |k*e         *lowergamma(k, l*z)
+    |-------------------------------  for z >= 0
+    <          Gamma(k + 1)
     |
-    |                                0                                  otherwise
+    |               0                 otherwise
     \
 
     >>> simplify(E(X))
@@ -775,7 +827,7 @@ def Erlang(name, k, l):
     .. [2] http://mathworld.wolfram.com/ErlangDistribution.html
     """
 
-    return rv(name, GammaDistribution, (k, 1/l))
+    return rv(name, GammaDistribution, (k, S.One/l))
 
 #-------------------------------------------------------------------------------
 # Exponential distribution -----------------------------------------------------
@@ -796,6 +848,15 @@ class ExponentialDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.expovariate(self.rate)
 
+    def _cdf(self, x):
+        return Piecewise(
+                (S.One - exp(-self.rate*x), x >= 0),
+                (0, True),
+        )
+
+    def _characteristic_function(self, t):
+        rate = self.rate
+        return rate / (rate - I*t)
 
 def Exponential(name, rate):
     r"""
@@ -868,6 +929,7 @@ def Exponential(name, rate):
 #-------------------------------------------------------------------------------
 # F distribution ---------------------------------------------------------------
 
+
 class FDistributionDistribution(SingleContinuousDistribution):
     _argnames = ('d1', 'd2')
 
@@ -878,6 +940,7 @@ class FDistributionDistribution(SingleContinuousDistribution):
         return (sqrt((d1*x)**d1*d2**d2 / (d1*x+d2)**(d1+d2))
                / (x * beta_fn(d1/2, d2/2)))
 
+
 def FDistribution(name, d1, d2):
     r"""
     Create a continuous random variable with a F distribution.
@@ -891,13 +954,11 @@ def FDistribution(name, d1, d2):
 
     with :math:`x > 0`.
 
-    .. TODO - What do these parameters mean?
-
     Parameters
     ==========
 
-    d1 : `d_1 > 0` a parameter
-    d2 : `d_2 > 0` a parameter
+    d1 : `d_1 > 0`, where d_1 is the degrees of freedom (n_1 - 1)
+    d2 : `d_2 > 0`, where d_2 is the degrees of freedom (n_2 - 1)
 
     Returns
     =======
@@ -923,9 +984,9 @@ def FDistribution(name, d1, d2):
       2    /       d1            -d1 - d2
     d2  *\/  (d1*z)  *(d1*z + d2)
     --------------------------------------
-                      /d1  d2\
-                z*beta|--, --|
-                      \2   2 /
+                    /d1  d2\
+                 z*B|--, --|
+                    \2   2 /
 
     References
     ==========
@@ -991,9 +1052,9 @@ def FisherZ(name, d1, d2):
         2    2  /    2*z     \           d1*z
     2*d1  *d2  *\d1*e    + d2/         *e
     -----------------------------------------
-                       /d1  d2\
-                   beta|--, --|
-                       \2   2 /
+                     /d1  d2\
+                    B|--, --|
+                     \2   2 /
 
     References
     ==========
@@ -1020,6 +1081,11 @@ class FrechetDistribution(SingleContinuousDistribution):
         a, s, m = self.a, self.s, self.m
         return a/s * ((x-m)/s)**(-1-a) * exp(-((x-m)/s)**(-a))
 
+    def _cdf(self, x):
+        a, s, m = self.a, self.s, self.m
+        return Piecewise((exp(-((x-m)/s)**(-a)), x >= m),
+                        (S.Zero, True))
+
 def Frechet(name, a, s=1, m=0):
     r"""
     Create a continuous random variable with a Frechet distribution.
@@ -1047,7 +1113,7 @@ def Frechet(name, a, s=1, m=0):
     Examples
     ========
 
-    >>> from sympy.stats import Frechet, density, E, std
+    >>> from sympy.stats import Frechet, density, E, std, cdf
     >>> from sympy import Symbol, simplify
 
     >>> a = Symbol("a", positive=True)
@@ -1060,6 +1126,9 @@ def Frechet(name, a, s=1, m=0):
     >>> density(X)(z)
     a*((-m + z)/s)**(-a - 1)*exp(-((-m + z)/s)**(-a))/s
 
+    >>> cdf(X)(z)
+     Piecewise((exp(-((-m + z)/s)**(-a)), m <= z), (0, True))
+
     References
     ==========
 
@@ -1089,6 +1158,12 @@ class GammaDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.gammavariate(self.k, self.theta)
 
+    def _cdf(self, x):
+        k, theta = self.k, self.theta
+        return Piecewise(
+                    (lowergamma(k, S(x)/theta)/gamma(k), x > 0),
+                    (S.Zero, True))
+
 
 def Gamma(name, k, theta):
     r"""
@@ -1131,17 +1206,17 @@ def Gamma(name, k, theta):
          -k  k - 1  theta
     theta  *z     *e
     ---------------------
-           gamma(k)
+           Gamma(k)
 
     >>> C = cdf(X, meijerg=True)(z)
     >>> pprint(C, use_unicode=False)
-    /                                   /     z  \
-    |                       k*lowergamma|k, -----|
-    |  k*lowergamma(k, 0)               \   theta/
-    <- ------------------ + ----------------------  for z >= 0
-    |     gamma(k + 1)           gamma(k + 1)
+    /            /     z  \
+    |k*lowergamma|k, -----|
+    |            \   theta/
+    <----------------------  for z >= 0
+    |     Gamma(k + 1)
     |
-    \                      0                        otherwise
+    \          0             otherwise
 
     >>> E(X)
     theta*gamma(k + 1)/gamma(k)
@@ -1164,6 +1239,7 @@ def Gamma(name, k, theta):
 #-------------------------------------------------------------------------------
 # Inverse Gamma distribution ---------------------------------------------------
 
+
 class GammaInverseDistribution(SingleContinuousDistribution):
     _argnames = ('a', 'b')
 
@@ -1178,6 +1254,19 @@ class GammaInverseDistribution(SingleContinuousDistribution):
         a, b = self.a, self.b
         return b**a/gamma(a) * x**(-a-1) * exp(-b/x)
 
+    def _cdf(self, x):
+        a, b = self.a, self.b
+        return Piecewise((uppergamma(a,b/x)/gamma(a), x > 0),
+                        (S.Zero, True))
+
+    def sample(self):
+        scipy = import_module('scipy')
+        if scipy:
+            from scipy.stats import invgamma
+            return invgamma.rvs(float(self.a), 0, float(self.b))
+        else:
+            raise NotImplementedError('Sampling the inverse Gamma Distribution requires Scipy.')
+
 def GammaInverse(name, a, b):
     r"""
     Create a continuous random variable with an inverse Gamma distribution.
@@ -1220,7 +1309,11 @@ def GammaInverse(name, a, b):
      a  -a - 1   z
     b *z      *e
     ---------------
-       gamma(a)
+       Gamma(a)
+
+    >>> cdf(X)(z)
+    Piecewise((uppergamma(a, b/z)/gamma(a), z > 0), (0, True))
+
 
     References
     ==========
@@ -1230,6 +1323,61 @@ def GammaInverse(name, a, b):
 
     return rv(name, GammaInverseDistribution, (a, b))
 
+#-------------------------------------------------------------------------------
+# Gumbel distribution --------------------------------------------------------
+
+
+class GumbelDistribution(SingleContinuousDistribution):
+    _argnames = ('beta', 'mu')
+
+    set = Interval(-oo, oo)
+
+    def pdf(self, x):
+        beta, mu = self.beta, self.mu
+        return (1/beta)*exp(-((x-mu)/beta)+exp(-((x-mu)/beta)))
+
+def Gumbel(name, beta, mu):
+    r"""
+    Create a Continuous Random Variable with Gumbel distribution.
+
+    The density of the Gumbel distribution is given by
+
+    .. math::
+        f(x) := \exp \left( -exp \left( x + \exp \left( -x \right) \right) \right)
+
+    with ::math 'x \in [ - \inf, \inf ]'.
+
+    Parameters
+    ==========
+
+    mu: Real number, 'mu' is a location
+    beta: Real number, 'beta > 0' is a scale
+
+    Returns
+    ==========
+
+    A RandomSymbol
+
+    Examples
+    ==========
+    >>> from sympy.stats import Gumbel, density, E, variance
+    >>> from sympy import Symbol, simplify, pprint
+    >>> x = Symbol("x")
+    >>> mu = Symbol("mu")
+    >>> beta = Symbol("beta", positive=True)
+    >>> X = Gumbel("x", beta, mu)
+    >>> density(X)(x)
+    exp(exp(-(-mu + x)/beta) - (-mu + x)/beta)/beta
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/GumbelDistribution.html
+    .. [2] https://en.wikipedia.org/wiki/Gumbel_distribution
+
+    """
+    return rv(name, GumbelDistribution, (beta, mu))
+
 #-------------------------------------------------------------------------------
 # Gompertz distribution --------------------------------------------------------
 
@@ -1247,6 +1395,7 @@ class GompertzDistribution(SingleContinuousDistribution):
         eta, b = self.eta, self.b
         return b*eta*exp(b*x)*exp(eta)*exp(-eta*exp(b*x))
 
+
 def Gompertz(name, b, eta):
     r"""
     Create a Continuous Random Variable with Gompertz distribution.
@@ -1295,6 +1444,7 @@ def Gompertz(name, b, eta):
 #-------------------------------------------------------------------------------
 # Kumaraswamy distribution -----------------------------------------------------
 
+
 class KumaraswamyDistribution(SingleContinuousDistribution):
     _argnames = ('a', 'b')
 
@@ -1309,6 +1459,14 @@ class KumaraswamyDistribution(SingleContinuousDistribution):
         a, b = self.a, self.b
         return a * b * x**(a-1) * (1-x**a)**(b-1)
 
+    def _cdf(self, x):
+        a, b = self.a, self.b
+        return Piecewise(
+            (S.Zero, x < S.Zero),
+            (1 - (1 - x**a)**b, x <= S.One),
+            (S.One, True))
+
+
 def Kumaraswamy(name, a, b):
     r"""
     Create a Continuous Random Variable with a Kumaraswamy distribution.
@@ -1334,7 +1492,7 @@ def Kumaraswamy(name, a, b):
     Examples
     ========
 
-    >>> from sympy.stats import Kumaraswamy, density, E, variance
+    >>> from sympy.stats import Kumaraswamy, density, E, variance, cdf
     >>> from sympy import Symbol, simplify, pprint
 
     >>> a = Symbol("a", positive=True)
@@ -1349,6 +1507,10 @@ def Kumaraswamy(name, a, b):
          a - 1 /   a    \
     a*b*z     *\- z  + 1/
 
+    >>> cdf(X)(z)
+    Piecewise((0, z < 0),
+            (-(-z**a + 1)**b + 1, z <= 1),
+            (1, True))
 
     References
     ==========
@@ -1369,6 +1531,13 @@ class LaplaceDistribution(SingleContinuousDistribution):
         mu, b = self.mu, self.b
         return 1/(2*b)*exp(-Abs(x - mu)/b)
 
+    def _cdf(self, x):
+        mu, b = self.mu, self.b
+        return Piecewise(
+                    (S.Half*exp((x - mu)/b), x < mu),
+                    (S.One - S.Half*exp(-(x - mu)/b), x >= mu)
+                        )
+
 
 def Laplace(name, mu, b):
     r"""
@@ -1393,7 +1562,7 @@ def Laplace(name, mu, b):
     Examples
     ========
 
-    >>> from sympy.stats import Laplace, density
+    >>> from sympy.stats import Laplace, density, cdf
     >>> from sympy import Symbol
 
     >>> mu = Symbol("mu")
@@ -1405,6 +1574,10 @@ def Laplace(name, mu, b):
     >>> density(X)(z)
     exp(-Abs(mu - z)/b)/(2*b)
 
+    >>> cdf(X)(z)
+    Piecewise((exp((-mu + z)/b)/2, mu > z),
+            (-exp((mu - z)/b)/2 + 1, True))
+
     References
     ==========
 
@@ -1425,6 +1598,12 @@ class LogisticDistribution(SingleContinuousDistribution):
         mu, s = self.mu, self.s
         return exp(-(x - mu)/s)/(s*(1 + exp(-(x - mu)/s))**2)
 
+    def _cdf(self, x):
+        mu, s = self.mu, self.s
+        return S.One/(1 + exp(-(x - mu)/s))
+
+    def _characteristic_function(self, t):
+        return Piecewise((exp(I*t*self.mu) * pi*self.s*t / sinh(pi*self.s*t), Ne(t, 0)), (S.One, True))
 
 def Logistic(name, mu, s):
     r"""
@@ -1449,7 +1628,7 @@ def Logistic(name, mu, s):
     Examples
     ========
 
-    >>> from sympy.stats import Logistic, density
+    >>> from sympy.stats import Logistic, density, cdf
     >>> from sympy import Symbol
 
     >>> mu = Symbol("mu", real=True)
@@ -1461,6 +1640,9 @@ def Logistic(name, mu, s):
     >>> density(X)(z)
     exp((mu - z)/s)/(s*(exp((mu - z)/s) + 1)**2)
 
+    >>> cdf(X)(z)
+    1/(exp((mu - z)/s) + 1)
+
     References
     ==========
 
@@ -1486,6 +1668,13 @@ class LogNormalDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.lognormvariate(self.mean, self.std)
 
+    def _cdf(self, x):
+        mean, std = self.mean, self.std
+        return Piecewise(
+                (S.Half + S.Half*erf((log(x) - mean)/sqrt(2)/std), x > 0),
+                (S.Zero, True)
+        )
+
 
 def LogNormal(name, mean, std):
     r"""
@@ -1628,6 +1817,12 @@ class NakagamiDistribution(SingleContinuousDistribution):
         mu, omega = self.mu, self.omega
         return 2*mu**mu/(gamma(mu)*omega**mu)*x**(2*mu - 1)*exp(-mu/omega*x**2)
 
+    def _cdf(self, x):
+        mu, omega = self.mu, self.omega
+        return Piecewise(
+                    (lowergamma(mu, (mu/omega)*x**2)/gamma(mu), x > 0),
+                    (S.Zero, True))
+
 
 def Nakagami(name, mu, omega):
     r"""
@@ -1655,7 +1850,7 @@ def Nakagami(name, mu, omega):
     Examples
     ========
 
-    >>> from sympy.stats import Nakagami, density, E, variance
+    >>> from sympy.stats import Nakagami, density, E, variance, cdf
     >>> from sympy import Symbol, simplify, pprint
 
     >>> mu = Symbol("mu", positive=True)
@@ -1672,7 +1867,7 @@ def Nakagami(name, mu, omega):
         mu      -mu  2*mu - 1  omega
     2*mu  *omega   *z        *e
     ----------------------------------
-                gamma(mu)
+                Gamma(mu)
 
     >>> simplify(E(X, meijerg=True))
     sqrt(mu)*sqrt(omega)*gamma(mu + 1/2)/gamma(mu + 1)
@@ -1680,9 +1875,14 @@ def Nakagami(name, mu, omega):
     >>> V = simplify(variance(X, meijerg=True))
     >>> pprint(V, use_unicode=False)
                         2
-             omega*gamma (mu + 1/2)
+             omega*Gamma (mu + 1/2)
     omega - -----------------------
-            gamma(mu)*gamma(mu + 1)
+            Gamma(mu)*Gamma(mu + 1)
+
+    >>> cdf(X)(z)
+    Piecewise((lowergamma(mu, mu*z**2/omega)/gamma(mu), z > 0),
+            (0, True))
+
 
     References
     ==========
@@ -1709,6 +1909,13 @@ class NormalDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.normalvariate(self.mean, self.std)
 
+    def _cdf(self, x):
+        mean, std = self.mean, self.std
+        return erf(sqrt(2)*(-mean + x)/(2*std))/2 + S.Half
+
+    def _characteristic_function(self, t):
+        mean, std = self.mean, self.std
+        return exp(I*mean*t - std**2*t**2/2)
 
 def Normal(name, mean, std):
     r"""
@@ -1799,6 +2006,13 @@ class ParetoDistribution(SingleContinuousDistribution):
     def sample(self):
         return random.paretovariate(self.alpha)
 
+    def _cdf(self, x):
+        xm, alpha = self.xm, self.alpha
+        return Piecewise(
+                (S.One - xm**alpha/x**alpha, x>=xm),
+                (0, True),
+        )
+
 
 def Pareto(name, xm, alpha):
     r"""
@@ -1849,6 +2063,7 @@ def Pareto(name, xm, alpha):
 #-------------------------------------------------------------------------------
 # QuadraticU distribution ------------------------------------------------------
 
+
 class QuadraticUDistribution(SingleContinuousDistribution):
     _argnames = ('a', 'b')
 
@@ -1864,6 +2079,7 @@ class QuadraticUDistribution(SingleContinuousDistribution):
                   (alpha * (x-beta)**2, And(a<=x, x<=b)),
                   (S.Zero, True))
 
+
 def QuadraticU(name, a, b):
     r"""
     Create a Continuous Random Variable with a U-quadratic distribution.
@@ -1904,7 +2120,7 @@ def QuadraticU(name, a, b):
     |   /  a   b    \
     |12*|- - - - + z|
     |   \  2   2    /
-    <-----------------  for And(a <= z, z <= b)
+    <-----------------  for And(b >= z, a <= z)
     |            3
     |    (-a + b)
     |
@@ -1921,6 +2137,7 @@ def QuadraticU(name, a, b):
 #-------------------------------------------------------------------------------
 # RaisedCosine distribution ----------------------------------------------------
 
+
 class RaisedCosineDistribution(SingleContinuousDistribution):
     _argnames = ('mu', 's')
 
@@ -1938,6 +2155,13 @@ class RaisedCosineDistribution(SingleContinuousDistribution):
                 ((1+cos(pi*(x-mu)/s)) / (2*s), And(mu-s<=x, x<=mu+s)),
                 (S.Zero, True))
 
+    def _characteristic_function(self, t):
+        mu, s = self.mu, self.s
+        return Piecewise((exp(-I*pi*mu/s)/2, Eq(t, -pi/s)),
+                         (exp(I*pi*mu/s)/2, Eq(t, pi/s)),
+                         (pi**2*sin(s*t)*exp(I*mu*t) / (s*t*(pi**2 - s**2*t**2)), True))
+
+
 def RaisedCosine(name, mu, s):
     r"""
     Create a Continuous Random Variable with a raised cosine distribution.
@@ -1977,7 +2201,7 @@ def RaisedCosine(name, mu, s):
     /   /pi*(-mu + z)\
     |cos|------------| + 1
     |   \     s      /
-    <---------------------  for And(z <= mu + s, mu - s <= z)
+    <---------------------  for And(z >= mu - s, z <= mu + s)
     |         2*s
     |
     \          0                        otherwise
@@ -2003,6 +2227,10 @@ class RayleighDistribution(SingleContinuousDistribution):
         sigma = self.sigma
         return x/sigma**2*exp(-x**2/(2*sigma**2))
 
+    def _characteristic_function(self, t):
+        sigma = self.sigma
+        return 1 - sigma*t*exp(-sigma**2*t**2/2) * sqrt(pi/2) * (erfi(sigma*t/sqrt(2)) - I)
+
 
 def Rayleigh(name, sigma):
     r"""
@@ -2057,6 +2285,7 @@ def Rayleigh(name, sigma):
 #-------------------------------------------------------------------------------
 # Shifted Gompertz distribution ------------------------------------------------
 
+
 class ShiftedGompertzDistribution(SingleContinuousDistribution):
     _argnames = ('b', 'eta')
 
@@ -2071,6 +2300,7 @@ class ShiftedGompertzDistribution(SingleContinuousDistribution):
         b, eta = self.b, self.eta
         return b*exp(-b*x)*exp(-eta*exp(-b*x))*(1+eta*(1-exp(-b*x)))
 
+
 def ShiftedGompertz(name, b, eta):
     r"""
     Create a continuous random variable with a Shifted Gompertz distribution.
@@ -2126,6 +2356,11 @@ class StudentTDistribution(SingleContinuousDistribution):
         nu = self.nu
         return 1/(sqrt(nu)*beta_fn(S(1)/2, nu/2))*(1 + x**2/nu)**(-(nu + 1)/2)
 
+    def _cdf(self, x):
+        nu = self.nu
+        return S.Half + x*gamma((nu+1)/2)*hyper((S.Half, (nu+1)/2),
+                                (S(3)/2,), -x**2/nu)/(sqrt(pi*nu)*gamma(nu/2))
+
 
 def StudentT(name, nu):
     r"""
@@ -2151,7 +2386,7 @@ def StudentT(name, nu):
     Examples
     ========
 
-    >>> from sympy.stats import StudentT, density, E, variance
+    >>> from sympy.stats import StudentT, density, E, variance, cdf
     >>> from sympy import Symbol, simplify, pprint
 
     >>> nu = Symbol("nu", positive=True)
@@ -2161,17 +2396,22 @@ def StudentT(name, nu):
 
     >>> D = density(X)(z)
     >>> pprint(D, use_unicode=False)
-                nu   1
-              - -- - -
-                2    2
-      /     2\
-      |    z |
-      |1 + --|
-      \    nu/
-    --------------------
-      ____     /     nu\
-    \/ nu *beta|1/2, --|
-               \     2 /
+               nu   1
+             - -- - -
+               2    2
+     /     2\
+     |    z |
+     |1 + --|
+     \    nu/
+    -----------------
+      ____  /     nu\
+    \/ nu *B|1/2, --|
+            \     2 /
+
+    >>> cdf(X)(z)
+    1/2 + z*gamma(nu/2 + 1/2)*hyper((1/2, nu/2 + 1/2), (3/2,),
+                                -z**2/nu)/(sqrt(pi)*sqrt(nu)*gamma(nu/2))
+
 
     References
     ==========
@@ -2182,6 +2422,86 @@ def StudentT(name, nu):
 
     return rv(name, StudentTDistribution, (nu, ))
 
+#-------------------------------------------------------------------------------
+# Trapezoidal distribution ------------------------------------------------------
+
+
+class TrapezoidalDistribution(SingleContinuousDistribution):
+    _argnames = ('a', 'b', 'c', 'd')
+
+    def pdf(self, x):
+        a, b, c, d = self.a, self.b, self.c, self.d
+        return Piecewise(
+            (2*(x-a) / ((b-a)*(d+c-a-b)), And(a <= x, x < b)),
+            (2 / (d+c-a-b), And(b <= x, x < c)),
+            (2*(d-x) / ((d-c)*(d+c-a-b)), And(c <= x, x <= d)),
+            (S.Zero, True))
+
+
+def Trapezoidal(name, a, b, c, d):
+    r"""
+    Create a continuous random variable with a trapezoidal distribution.
+
+    The density of the trapezoidal distribution is given by
+
+    .. math::
+        f(x) := \begin{cases}
+                  0 & \mathrm{for\ } x < a, \\
+                  \frac{2(x-a)}{(b-a)(d+c-a-b)} & \mathrm{for\ } a \le x < b, \\
+                  \frac{2}{d+c-a-b} & \mathrm{for\ } b \le x < c, \\
+                  \frac{2(d-x)}{(d-c)(d+c-a-b)} & \mathrm{for\ } c \le x < d, \\
+                  0 & \mathrm{for\ } d < x.
+                \end{cases}
+
+    Parameters
+    ==========
+
+    a : Real number, :math:`a < d`
+    b : Real number, :math:`a <= b < c`
+    c : Real number, :math:`b < c <= d`
+    d : Real number
+
+    Returns
+    =======
+
+    A RandomSymbol.
+
+    Examples
+    ========
+
+    >>> from sympy.stats import Trapezoidal, density, E
+    >>> from sympy import Symbol, pprint
+
+    >>> a = Symbol("a")
+    >>> b = Symbol("b")
+    >>> c = Symbol("c")
+    >>> d = Symbol("d")
+    >>> z = Symbol("z")
+
+    >>> X = Trapezoidal("x", a,b,c,d)
+
+    >>> pprint(density(X)(z), use_unicode=False)
+    /        -2*a + 2*z
+    |-------------------------  for And(a <= z, b > z)
+    |(-a + b)*(-a - b + c + d)
+    |
+    |           2
+    |     --------------        for And(b <= z, c > z)
+    <     -a - b + c + d
+    |
+    |        2*d - 2*z
+    |-------------------------  for And(d >= z, c <= z)
+    |(-c + d)*(-a - b + c + d)
+    |
+    \            0                     otherwise
+
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Trapezoidal_distribution
+    """
+    return rv(name, TrapezoidalDistribution, (a, b, c, d))
+
 #-------------------------------------------------------------------------------
 # Triangular distribution ------------------------------------------------------
 
@@ -2240,15 +2560,15 @@ def Triangular(name, a, b, c):
 
     >>> pprint(density(X)(z), use_unicode=False)
     /    -2*a + 2*z
-    |-----------------  for And(a <= z, z < c)
+    |-----------------  for And(a <= z, c > z)
     |(-a + b)*(-a + c)
     |
     |       2
-    |     ------              for z = c
+    |     ------              for c = z
     <     -a + b
     |
     |   2*b - 2*z
-    |----------------   for And(z <= b, c < z)
+    |----------------   for And(b >= z, c < z)
     |(-a + b)*(b - c)
     |
     \        0                otherwise
@@ -2273,18 +2593,21 @@ class UniformDistribution(SingleContinuousDistribution):
         left, right = self.left, self.right
         return Piecewise(
             (S.One/(right - left), And(left <= x, x <= right)),
-            (S.Zero, True))
+            (S.Zero, True)
+        )
 
-    def compute_cdf(self, **kwargs):
-        from sympy import Lambda, Min
-        z = Dummy('z', real=True, finite=True)
-        result = SingleContinuousDistribution.compute_cdf(self, **kwargs)(z)
-        reps = {
-            Min(z, self.right): z,
-            Min(z, self.left, self.right): self.left,
-            Min(z, self.left): self.left}
-        result = result.subs(reps)
-        return Lambda(z, result)
+    def _cdf(self, x):
+        left, right = self.left, self.right
+        return Piecewise(
+            (S.Zero, x < left),
+            ((x - left)/(right - left), x <= right),
+            (S.One, True)
+        )
+
+    def _characteristic_function(self, t):
+        left, right = self.left, self.right
+        return Piecewise(((exp(I*t*right) - exp(I*t*left)) / (I*t*(right - left)), Ne(t, 0)),
+                         (S.One, True))
 
     def expectation(self, expr, var, **kwargs):
         from sympy import Max, Min
@@ -2336,7 +2659,7 @@ def Uniform(name, left, right):
     >>> X = Uniform("x", a, b)
 
     >>> density(X)(z)
-    Piecewise((1/(-a + b), And(a <= z, z <= b)), (0, True))
+    Piecewise((1/(-a + b), (b >= z) & (a <= z)), (0, True))
 
     >>> cdf(X)(z)  # doctest: +SKIP
     -a/(-a + b) + z/(-a + b)
@@ -2373,7 +2696,13 @@ class UniformSumDistribution(SingleContinuousDistribution):
         return 1/factorial(
             n - 1)*Sum((-1)**k*binomial(n, k)*(x - k)**(n - 1), (k, 0, floor(x)))
 
-
+    def _cdf(self, x):
+        n = self.n
+        k = Dummy("k")
+        return Piecewise((S.Zero, x < 0),
+                        (1/factorial(n)*Sum((-1)**k*binomial(n, k)*(x - k)**(n),
+                        (k, 0, floor(x))), x <= n),
+                        (S.One, True))
 
 def UniformSum(name, n):
     r"""
@@ -2401,7 +2730,7 @@ def UniformSum(name, n):
     Examples
     ========
 
-    >>> from sympy.stats import UniformSum, density
+    >>> from sympy.stats import UniformSum, density, cdf
     >>> from sympy import Symbol, pprint
 
     >>> n = Symbol("n", integer=True)
@@ -2422,6 +2751,18 @@ def UniformSum(name, n):
     --------------------------------
                 (n - 1)!
 
+    >>> cdf(X)(z)
+    Piecewise((0, z < 0), (Sum((-1)**_k*(-_k + z)**n*binomial(n, _k),
+                    (_k, 0, floor(z)))/factorial(n), n >= z), (1, True))
+
+
+    Compute cdf with specific 'x' and 'n' values as follows :
+    >>> cdf(UniformSum("x", 5), evaluate=False)(2).doit()
+    9/40
+
+    The argument evaluate=False prevents an attempt at evaluation
+    of the sum for general n, before the argument 2 is passed.
+
     References
     ==========
 
@@ -2434,6 +2775,7 @@ def UniformSum(name, n):
 #-------------------------------------------------------------------------------
 # VonMises distribution --------------------------------------------------------
 
+
 class VonMisesDistribution(SingleContinuousDistribution):
     _argnames = ('mu', 'k')
 
@@ -2591,6 +2933,9 @@ class WignerSemicircleDistribution(SingleContinuousDistribution):
         R = self.R
         return 2/(pi*R**2)*sqrt(R**2 - x**2)
 
+    def _characteristic_function(self, t):
+        return Piecewise((2 * besselj(1, self.R*t) / (self.R*t), Ne(t, 0)),
+                         (S.One, True))
 
 def WignerSemicircle(name, R):
     r"""
